/*
* File Name : lib/ParameterError.js
* Purpose : This file contains the ParameterError declaration
* Creation Date : 2014/05/24
* Last Modified : sam. 24 mai 2014 22:53:36 CEST
* Created By : Alexandre Chaussier, infOpen <a.chaussier@infopen.pro>
* License : GPLv3

This module is used to manage custom errors on infOpen programs.
Copyright (c) 2014 "Alexandre Chaussier, infOpen"
infOpen - IT company for small and medium companies [http://infopen.pro]

This file is part of infopen-errors.

infopen-errors is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>, or
write to the Free Software Foundation, Inc., 51 Franklin Street,
Fifth Floor, Boston, MA 02110-1301  USA.
*/

'use strict';

// Import dependencies
var inherits = require('util').inherits;
var InfopenError = require('../InfopenError');

var ParameterError = function ParameterError(param, category, severity,
                       source, value, expected, moreInfos) {

  /*
   * Private variables
   */

  /** Object used to indexed function to execute depending error category */
  var _errorMessages = {};
  /** Variable will stock generated error message */
  var _msg;


  /*
   * Private methods
   */

  /**
   * This function generate the error message to return for bad value param
   * @param {string} param - Parameter which caused error
   * @param {string} source - Source of error
   * @param {*} value - Value of param
   * @param {*} expected - Expected value for param
   * @return {string} Return the error message
   */
  function _generateBadValueParamMessage(param, source, value, expected) {

    var errorMsg;
    errorMsg = 'Bad value for "' + param + '" parameter in "' + source + '".';
    errorMsg += ' Expected : "' + expected + '" but received "' + value + '"';

    return errorMsg;
  }
  _errorMessages.badValue = _generateBadValueParamMessage;


  /**
   * This function generate the error message to return for bad type param
   * @param {string} param - Parameter which caused error
   * @param {string} source - Source of error
   * @param {*} value - Value of param
   * @param {*} expected - Expected value for param
   * @return {string} Return the error message
   */
  function _generateBadTypeParamMessage(param, source, value, expected) {

    var errorMsg;
    errorMsg = 'Bad type for "' + param + '" parameter in "' + source + '".';
    errorMsg += ' Expected : "' + expected + '" but received "' + value + '"';

    return errorMsg;
  }
  _errorMessages.badType = _generateBadTypeParamMessage;


  /**
   * This function generate the error message to return for mandatory param
   * which is not sent
   * @param {string} param - Parameter which caused error
   * @param {string} source - Source of error
   * @param {*} value - Value of param
   * @param {*} expected - Expected value for param
   * @return {string} Return the error message
   */
  function _generateMandatoryNotSentParamMessage(param, source, value, expected) {

    var errorMsg;
    errorMsg = '"' + param + '" parameter in "' + source + '" is mandatory !';

    return errorMsg;
  }
  _errorMessages.mandatoryNotSent = _generateMandatoryNotSentParamMessage;


  /**
   * @desc This function generate the error message to return
   * @param {string} param - Parameter which caused error
   * @param {string} category - Category of error
   * @param {string} source - Source of error
   * @param {*} value - Value of param
   * @param {*} expected - Expected value for param
   * @return {string} Return the error message
   */
  function _generateErrorMessage(param, category, source, value, expected) {

    var errorMsg;
    var getErrorMessage;

    if (_errorMessages.hasOwnProperty(category)) {
      getErrorMessage = _errorMessages[category];
      return getErrorMessage(param, source, value, expected);
    } else {
      errorMsg =  'Unknown parameter error on "' + source;
      errorMsg += '" for parameter "' + param + '" with value "' + value + '"';
      return errorMsg;
    }
  }
  

  /**
   * @constructor ParameterError
   * @desc This function define the ParameterError object
   * @inherits InfopenError
   * @param {string} param - Parameter which caused error
   * @param {string} category - Category of error
   * @param {string} source - Source of error
   * @param {*} value - Value of param
   * @param {*} expected - Expected value for param
   * @return {ParameterError} Return a new instance of ErrorParameter object
   */
  _msg = _generateErrorMessage(param, category, source, value, expected);

  InfopenError.call(this, _msg);
  Error.captureStackTrace(this, ParameterError);

  this.category  = category;
  this.message   = _msg;
  this.parameter = param;
  this.source    = source;
  this.severity  = severity;
  this.name      = 'ParameterError';

  if (moreInfos !== undefined) {
    this.moreInfos = moreInfos;
  }

  return this;
};
inherits(ParameterError, InfopenError);

module.exports = ParameterError;
